@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Optional, TYPE_CHECKING, TypeVar, List, Type, Generator
+from typing import Optional, TYPE_CHECKING, List
 from typing_extensions import Protocol
 
 from poetry.poetry import Poetry
@@ -11,9 +11,6 @@ if TYPE_CHECKING:
     from coveo_stew.metadata.poetry_api import PoetryAPI
 
 
-T_PythonProject = TypeVar("T_PythonProject", bound="PythonProjectAPI")
-
-
 class PythonProjectAPI(Protocol):
     verbose: bool
     project_path: Path
@@ -33,7 +30,13 @@ class PythonProjectAPI(Protocol):
     ) -> List["PythonEnvironment"]:
         ...
 
-    def install(self, *, remove_untracked: bool = True, quiet: bool = False) -> None:
+    def install(
+        self,
+        *,
+        environment: "PythonEnvironment" = None,
+        remove_untracked: bool = True,
+        quiet: bool = False
+    ) -> None:
         ...
 
     def build(self, target_path: Path) -> Path:
@@ -42,24 +45,3 @@ class PythonProjectAPI(Protocol):
     @property
     def lock_is_outdated(self) -> bool:
         ...
-
-    @classmethod
-    def find_pyproject(
-        cls: Type[T_PythonProject], project_name: str, path: Path = None, *, verbose: bool = False
-    ) -> T_PythonProject:
-        ...
-
-    @classmethod
-    def find_pyprojects(
-        cls: Type[T_PythonProject],
-        path: Path = None,
-        *,
-        query: str = None,
-        exact_match: bool = False,
-        verbose: bool = False
-    ) -> Generator[T_PythonProject, None, None]:
-        ...
-
-    @classmethod
-    def find_pyproject_paths(cls, path: Path) -> Generator[Path, None, None]:
-        ...
