@@ -1,12 +1,14 @@
 """Interact with python projects programmatically."""
 
+from contextlib import contextmanager
 import os
 from pathlib import Path
 import re
 import shutil
 from shutil import rmtree
 import sys
-from typing import Generator, Optional, Type, Any, List
+from typing import Generator, Optional, Any, List, Tuple
+import warnings
 
 from coveo_functools.casing import flexfactory
 from coveo_itertools.lookups import dict_lookup
@@ -18,10 +20,10 @@ from poetry.factory import Factory
 from coveo_stew.ci.config import ContinuousIntegrationConfig
 from coveo_stew.ci.runner import RunnerStatus
 from coveo_stew.environment import PythonEnvironment, coveo_stew_environment, PythonTool
-from coveo_stew.exceptions import PythonProjectNotFound, PythonProjectException
+from coveo_stew.exceptions import PythonProjectException, NotAPoetryProject
 from coveo_stew.metadata.stew_api import StewPackage
 from coveo_stew.metadata.poetry_api import PoetryAPI
-from coveo_stew.metadata.pyproject_api import PythonProjectAPI, T_PythonProject
+from coveo_stew.metadata.pyproject_api import PythonProjectAPI
 from coveo_stew.metadata.python_api import PythonFile
 from coveo_stew.utils import load_toml_from_path
 
@@ -39,9 +41,13 @@ class PythonProject(PythonProjectAPI):
 
         toml_content = load_toml_from_path(self.toml_path)
 
-        self.package: PoetryAPI = flexfactory(
-            PoetryAPI, **dict_lookup(toml_content, "tool", "poetry"), _pyproject=self
-        )
+        try:
+            self.package: PoetryAPI = flexfactory(
+                PoetryAPI, **dict_lookup(toml_content, "tool", "poetry"), _pyproject=self
+            )
+        except KeyError as exception:
+            raise NotAPoetryProject from exception
+
         self.egg_path: Path = self.project_path / f"{self.package.safe_name}.egg-info"
 
         self.options: StewPackage = flexfactory(
@@ -71,36 +77,32 @@ class PythonProject(PythonProjectAPI):
             repo_root = None
 
         self.repo_root: Optional[Path] = repo_root
+        self._cached_activated_environment: Optional[PythonEnvironment] = None
 
     def relative_path(self, path: Path) -> Path:
         """returns the relative path of a path vs the project folder."""
         return path.relative_to(self.project_path)
 
-    @classmethod
-    def find_pyproject_paths(cls, path: Path) -> Generator[Path, None, None]:
-        """Subclasses may override this to change/filter discovery."""
-        yield from path.rglob(f"{PythonFile.PyProjectToml}")
-
     @classmethod
     def find_pyproject(
-        cls: Type[T_PythonProject], project_name: str, path: Path = None, *, verbose: bool = False
-    ) -> T_PythonProject:
-        project = next(
-            cls.find_pyprojects(path, query=project_name, exact_match=True, verbose=verbose), None
+        cls, project_name: str, path: Path = None, *, verbose: bool = False
+    ) -> "PythonProject":
+        warnings.warn(
+            "This functionality moved to the `coveo_stew.discovery` module.", DeprecationWarning
         )
-        if not project:
-            raise PythonProjectNotFound(f"{project_name} cannot be found in {path}")
-        return project
+        from coveo_stew.discovery import find_pyproject
+
+        return find_pyproject(project_name, path, verbose=verbose)
 
     @classmethod
     def find_pyprojects(
-        cls: Type[T_PythonProject],
+        cls,
         path: Path = None,
         *,
         query: str = None,
         exact_match: bool = False,
         verbose: bool = False,
-    ) -> Generator[T_PythonProject, None, None]:
+    ) -> Generator["PythonProject", None, None]:
         """Factory; scan a path (recursive) and return a PythonProject instance for each pyproject.toml
 
         Parameters:
@@ -109,35 +111,12 @@ class PythonProject(PythonProjectAPI):
             exact_match: turns query into an exact match (except for - and _). Recommended use: CI scripts
             verbose: output more details to command line
         """
-        if not path:
-            path = find_repo_root(default=".")
+        warnings.warn(
+            "This functionality moved to the `coveo_stew.discovery` module.", DeprecationWarning
+        )
+        from coveo_stew.discovery import discover_pyprojects
 
-        if exact_match and not query:
-            raise PythonProjectNotFound("An exact match was requested but no query was provided.")
-
-        count_projects = 0
-        for poetry_file in cls.find_pyproject_paths(path):
-            project = cls(poetry_file, verbose=verbose)
-            if verbose:
-                echo.noise("PyProject found: ", project)
-
-            if (
-                not query
-                or (exact_match and project.package.name == query)
-                or (
-                    not exact_match
-                    and query.replace("-", "_").lower() in project.package.safe_name.lower()
-                )
-            ):
-                count_projects += 1
-                yield project
-
-        if count_projects == 0:
-            raise PythonProjectNotFound(
-                f"Cannot find any project that could match {query}"
-                if query
-                else "No python projects were found."
-            )
+        yield from discover_pyprojects(path, query=query, exact_match=exact_match, verbose=verbose)
 
     @property
     def lock_is_outdated(self) -> bool:
@@ -146,6 +125,25 @@ class PythonProject(PythonProjectAPI):
             return False
         return not self.poetry.locker.is_fresh()
 
+    @property
+    def activated_environment(self) -> Optional[PythonEnvironment]:
+        """The environment activated for a project.
+
+        Note: cached for performance, could theoretically become out-of-sync with reality, either due to a bug,
+        or because someone called `poetry env use` while we were working :shrug:
+
+        To prevent out of sync:
+            - Use the context manager `self._activate_poetry_environment`
+            - or use the `environment` argument of `self.poetry_run`
+        """
+        if self._cached_activated_environment is None:
+            activated = self.poetry_run(
+                "env", "info", "--path", capture_output=True, breakout_of_venv=True
+            ).strip()
+            if activated:
+                self._cached_activated_environment = PythonEnvironment(activated)
+        return self._cached_activated_environment
+
     def virtual_environments(
         self, *, create_default_if_missing: bool = False
     ) -> List[PythonEnvironment]:
@@ -164,8 +162,8 @@ class PythonProject(PythonProjectAPI):
                 environments.append(PythonEnvironment(path))
 
         if not environments and create_default_if_missing:
-            # when we call `poetry run <cmd>` on a project, poetry will create the environment if it doesn't exist.
-            self.poetry_run("run", "python", "--version")
+            # install the environment using poetry's default by calling `poetry install`
+            self.install()
             environments = self.virtual_environments()
 
         return environments
@@ -223,13 +221,19 @@ class PythonProject(PythonProjectAPI):
 
         return target
 
-    def launch_continuous_integration(self, auto_fix: bool = False) -> bool:
+    def launch_continuous_integration(
+        self, auto_fix: bool = False, checks: List[str] = None
+    ) -> bool:
         """Launch all continuous integration runners on the project."""
         if self.ci.disabled:
             return True
 
+        checks = [check.lower() for check in checks or []]
         exceptions: List[DetailedCalledProcessError] = []
         for runner in self.ci.runners:
+            if checks and runner.name.lower() not in checks:
+                continue
+
             for environment in self.virtual_environments(create_default_if_missing=True):
                 try:
                     echo.normal(
@@ -256,18 +260,28 @@ class PythonProject(PythonProjectAPI):
                 echo.warning(f"{len(exceptions)} exceptions found; raising first one.")
             raise exceptions[0]
 
-        return all(runner.status is RunnerStatus.Success for runner in self.ci.runners)
+        allowed_statuses: Tuple[RunnerStatus, ...] = (
+            (RunnerStatus.Success, RunnerStatus.NotRan) if checks else (RunnerStatus.Success,)
+        )
+        return all(runner.status in allowed_statuses for runner in self.ci.runners)
 
-    def install(self, remove_untracked: bool = True, quiet: bool = False) -> None:
+    def install(
+        self,
+        *,
+        environment: PythonEnvironment = None,
+        remove_untracked: bool = True,
+        quiet: bool = False,
+    ) -> None:
         """
         Performs a 'poetry install --remove-untracked' on the project. If an environment is provided, target it.
         """
         command = ["install"]
         if remove_untracked:
             command.append("--remove-untracked")
-        if quiet:
+        if quiet and not self.verbose:
             command.append("--quiet")
-        self.poetry_run(*command)
+
+        self.poetry_run(*command, environment=environment)
 
     def remove_egg_info(self) -> bool:
         """Removes the egg-info (editable project hook) from the folder. Returns True if we removed it."""
@@ -291,7 +305,11 @@ class PythonProject(PythonProjectAPI):
         return False
 
     def poetry_run(
-        self, *commands: Any, capture_output: bool = False, breakout_of_venv: bool = True
+        self,
+        *commands: Any,
+        capture_output: bool = False,
+        breakout_of_venv: bool = True,
+        environment: PythonEnvironment = None,
     ) -> Optional[str]:
         """internal run-a-poetry-command."""
         # we use the poetry executable from our dependencies, not from the project's environment!
@@ -305,13 +323,38 @@ class PythonProject(PythonProjectAPI):
         if breakout_of_venv:
             environment_variables.pop("VIRTUAL_ENV", None)
 
-        return check_run(
-            *poetry_env.build_command(PythonTool.Poetry, *commands, "-vv" if self.verbose else ""),
-            working_directory=self.project_path,
-            capture_output=capture_output,
-            verbose=self.verbose,
-            env=environment_variables,
-        )
+        with self._activate_poetry_environment(environment):
+            return check_run(
+                *poetry_env.build_command(
+                    PythonTool.Poetry, *commands, "-vv" if self.verbose else ""
+                ),
+                working_directory=self.project_path,
+                capture_output=capture_output,
+                verbose=self.verbose,
+                env=environment_variables,
+            )
+
+    @contextmanager
+    def _activate_poetry_environment(
+        self, environment: PythonEnvironment = None
+    ) -> Generator[None, None, None]:
+        """ Context manager that can be used to run a block of code in a particular environment. """
+        if not environment:
+            # `self.activated_environment` uses us without an environment specified; prevents infinite recursion
+            yield
+            return
+
+        current_environment = self.activated_environment
+        if current_environment == environment:
+            yield
+            return
+
+        try:
+            self.poetry_run("env", "use", environment.python_executable)
+            yield
+        finally:
+            if current_environment:
+                self.poetry_run("env", "use", current_environment.python_executable)
 
     def __str__(self) -> str:
         return f"{self.package.name} [{self.toml_path}]"
